/** 
 * @file llmemorystream.cpp
 * @author Phoenix
 * @date 2005-06-03
 * @brief Buffer and stream for a fixed linear memory segment.
 *
 * $LicenseInfo:firstyear=2005&license=viewergpl$
 * 
 * Copyright (c) 2005-2009, Linden Research, Inc.
 * 
 * Second Life Viewer Source Code
 * The source code in this file ("Source Code") is provided by Linden Lab
 * to you under the terms of the GNU General Public License, version 2.0
 * ("GPL"), unless you have obtained a separate licensing agreement
 * ("Other License"), formally executed by you and Linden Lab.  Terms of
 * the GPL can be found in doc/GPL-license.txt in this distribution, or
 * online at http://secondlifegrid.net/programs/open_source/licensing/gplv2
 * 
 * There are special exceptions to the terms and conditions of the GPL as
 * it is applied to this Source Code. View the full text of the exception
 * in the file doc/FLOSS-exception.txt in this software distribution, or
 * online at
 * http://secondlifegrid.net/programs/open_source/licensing/flossexception
 * 
 * By copying, modifying or distributing this software, you acknowledge
 * that you have read and understood your obligations described above,
 * and agree to abide by those obligations.
 * 
 * ALL LINDEN LAB SOURCE CODE IS PROVIDED "AS IS." LINDEN LAB MAKES NO
 * WARRANTIES, EXPRESS, IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY,
 * COMPLETENESS OR PERFORMANCE.
 * $/LicenseInfo$
 */

#include "linden_common.h"
#include "llmemorystream.h"

LLMemoryStreamBuf::LLMemoryStreamBuf(const U8* start, S32 length)
{
	reset(start, length);
}

LLMemoryStreamBuf::~LLMemoryStreamBuf()
{
}

void LLMemoryStreamBuf::reset(const U8* start, S32 length)
{
	setg((char*)start, (char*)start, (char*)start + length);
}

int LLMemoryStreamBuf::underflow()
{
	//LL_DEBUGS() << "LLMemoryStreamBuf::underflow()" << LL_ENDL;
	if(gptr() < egptr())
	{
		return *gptr();
	}
	return EOF;
}

/** 
 * @class LLMemoryStreamBuf
 */

LLMemoryStream::LLMemoryStream(const U8* start, S32 length) :
	std::istream(&mStreamBuf),
	mStreamBuf(start, length)
{
}

LLMemoryStream::~LLMemoryStream()
{
}


