/** 
 * @file llpanelcontents.h
 * @brief Object contents panel in the tools floater.
 *
 * $LicenseInfo:firstyear=2001&license=viewergpl$
 * 
 * Copyright (c) 2001-2009, Linden Research, Inc.
 * 
 * Second Life Viewer Source Code
 * The source code in this file ("Source Code") is provided by Linden Lab
 * to you under the terms of the GNU General Public License, version 2.0
 * ("GPL"), unless you have obtained a separate licensing agreement
 * ("Other License"), formally executed by you and Linden Lab.  Terms of
 * the GPL can be found in doc/GPL-license.txt in this distribution, or
 * online at http://secondlifegrid.net/programs/open_source/licensing/gplv2
 * 
 * There are special exceptions to the terms and conditions of the GPL as
 * it is applied to this Source Code. View the full text of the exception
 * in the file doc/FLOSS-exception.txt in this software distribution, or
 * online at
 * http://secondlifegrid.net/programs/open_source/licensing/flossexception
 * 
 * By copying, modifying or distributing this software, you acknowledge
 * that you have read and understood your obligations described above,
 * and agree to abide by those obligations.
 * 
 * ALL LINDEN LAB SOURCE CODE IS PROVIDED "AS IS." LINDEN LAB MAKES NO
 * WARRANTIES, EXPRESS, IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY,
 * COMPLETENESS OR PERFORMANCE.
 * $/LicenseInfo$
 */

#ifndef LL_LLPANELCONTENTS_H
#define LL_LLPANELCONTENTS_H

#include "v3math.h"
#include "llpanel.h"

class LLButton;
class LLPanelObjectInventory;
class LLViewerObject;
class LLCheckBoxCtrl;
class LLSpinCtrl;

class LLPanelContents : public LLPanel
{
public:
	virtual	BOOL postBuild();
	LLPanelContents(const std::string& name);
	virtual ~LLPanelContents();

	void			refresh();

	static void		onClickNewScript(		void* userdata);
	static void		onClickPermissions(		void* userdata);

    // Key suffix for "tentative" fields
    static const char* TENTATIVE_SUFFIX;

    // These aren't fields in LLMediaEntry, so we have to define them ourselves for checkbox control
    static const char* PERMS_OWNER_INTERACT_KEY;
    static const char* PERMS_OWNER_CONTROL_KEY;
    static const char* PERMS_GROUP_INTERACT_KEY;
    static const char* PERMS_GROUP_CONTROL_KEY;
    static const char* PERMS_ANYONE_INTERACT_KEY;
    static const char* PERMS_ANYONE_CONTROL_KEY;

protected:
	void			getState(LLViewerObject *object);

public:
	LLPanelObjectInventory* mPanelInventory;
};

#endif
