/** 
 * @file llpreviewlandmark.h
 * @brief LLPreviewLandmark class implementation
 *
 * $LicenseInfo:firstyear=2002&license=viewergpl$
 * 
 * Copyright (c) 2002-2009, Linden Research, Inc.
 * 
 * Second Life Viewer Source Code
 * The source code in this file ("Source Code") is provided by Linden Lab
 * to you under the terms of the GNU General Public License, version 2.0
 * ("GPL"), unless you have obtained a separate licensing agreement
 * ("Other License"), formally executed by you and Linden Lab.  Terms of
 * the GPL can be found in doc/GPL-license.txt in this distribution, or
 * online at http://secondlifegrid.net/programs/open_source/licensing/gplv2
 * 
 * There are special exceptions to the terms and conditions of the GPL as
 * it is applied to this Source Code. View the full text of the exception
 * in the file doc/FLOSS-exception.txt in this software distribution, or
 * online at
 * http://secondlifegrid.net/programs/open_source/licensing/flossexception
 * 
 * By copying, modifying or distributing this software, you acknowledge
 * that you have read and understood your obligations described above,
 * and agree to abide by those obligations.
 * 
 * ALL LINDEN LAB SOURCE CODE IS PROVIDED "AS IS." LINDEN LAB MAKES NO
 * WARRANTIES, EXPRESS, IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY,
 * COMPLETENESS OR PERFORMANCE.
 * $/LicenseInfo$
 */

#ifndef LL_LLPREVIEWLANDMARK_H
#define LL_LLPREVIEWLANDMARK_H

#include "llpreview.h"

class LLIconCtrl;
class LLPanelPlace;
class LLLandmark;

class LLPreviewLandmark final : public LLPreview
{
public:
	LLPreviewLandmark(const std::string& name, const LLRect& rect, const std::string& title,
					  const LLUUID& item_uuid,
					  BOOL show_keep_discard = FALSE,
					  LLViewerInventoryItem* inv_item = nullptr);
	virtual ~LLPreviewLandmark();

	/*virtual*/ void			draw() override;

	const std::string&				getName() const;
	const LLColor4&				getMarkerColor() const;

	LLVector3d getPositionGlobal() const;

	static void* createPlaceDetail(void* userdata);

	/*virtual*/ void loadAsset() override;
	/*virtual*/ EAssetStatus getAssetStatus() override;

protected:
	void getDegreesAndDist(F32* degrees, F64* horiz_dist, F64* vert_dist) const;

	const char *getTitleName() const override { return "Landmark"; }

private:
	LLIconCtrl*	mIconLandmark;
	LLPanelPlace* mPlacePanel;
	LLLandmark* mLandmark;
	LLColor4 mMarkerColor;
};

#endif
